% Units are inches, lbs

% General options
width           = 120;
height          = 144;
g               = 386;  % in/sec^2
weight          = 18000; % lbs
mass            = weight / g;
mass_divisor    = [.01 .01];
yield_scale     = 1;     % yield stress will be multiplied by this number

% Options governing the applied force
% f(t) = amplitude * sin( frequency * t), min_time < t < max_time
min_time        =  0;
max_time        =  15;
amplitude       = 50000;  % lbs
frequency       = 75;     % rad/sec

% These are used for the periodic update information
global time_stamp;
global time_stamp_dt;

time_stamp = 1;        % first output
time_stamp_dt = 1;  % output every dt seconds thereafter

% Steel 36 ksi
data.materials(1).E     = 29000000;
data.materials(1).G     = 10440000;
data.materials(1).Fy    = 36000 * yield_scale;
data.materials(1).name  = '36ksi';
data.materials(1).alpha = 0;


% W14x109 (columns)
data.sections(1).A=32.0;
data.sections(1).I=1240;
data.sections(1).name = 'W14x109';

% W24x068 (beam)
data.sections(2).A=20.1;
data.sections(2).I=1830;
data.sections(2).Z=177.0;
data.sections(2).name = 'W24x68';

data.nodelist = [ ...
	    0,      0 ; ...
	    0, height ; ...
	width, height ; ...
	width,      0];

data.elements(1).material = 1;
data.elements(1).section  = 1;
data.elements(1).nodes    = [ 1 2];
data.elements(1).type     = 'beam2d';

data.elements(2) = data.elements(1);
data.elements(2).nodes    = [ 4 3];

data.elements(3).material = 1;
data.elements(3).section  = 2;
data.elements(3).nodes    = [ 2 3];
data.elements(3).type     = 'nlbeam2d';

% First and fourth node fixed in all dofs, 2,3 fixed in y-direction
data.boundary_conditions = [ 1 1 1; 0 1 0; 0 1 0; 1 1 1 ];

% This should probably be done automatically somehow
[data.dofs_pos, data.dofs_num, data.num_dofs] = findDOFs (data.elements);
data.num_nodes = size(data.nodelist, 1);
data.num_elements = length(data.elements);

k = assemble( data, 'stiffness-matrix');
[K, dof_map] = apply_bc( k, data, 'remove');
clear k
% Linear System
% K =
%   5.0020e+06   1.0405e+07  -4.8575e+06            0
%   1.0405e+07   2.7679e+09            0   8.8450e+08
%  -4.8575e+06            0   5.0020e+06   1.0405e+07
%            0   8.8450e+08   1.0405e+07   2.7679e+09

% This T matrix constrains nodes 2 and 3 to have the same displacement
T_constrain = [ 1  0  0; 0  1  0; 1  0  0; 0  0  1];

% data.K is the constrained stiffness matrix:
% [ F  ]          [  X  ]
% [ M2 ] = data.K [ th2 ]
% [ M3 ]          [ th3 ]
%
% where th2 and th3 are the rotations (theta_2, theta_3) at the top nodes
data.K = T_constrain' * K * T_constrain;

% Get the pre-yield stiffness for modal analysis, etc.
data.pre_yield_stiffness = 1;  % this is deleted later
pre_yield_K = assemble( data, 'stiffness-matrix');
[pre_yield_K, dof_map] = apply_bc( pre_yield_K, data, 'remove');
data.pre_yield_K = T_constrain' * pre_yield_K * T_constrain;  %'
clear pre_yield_K
data = rmfield( data, 'pre_yield_stiffness');


% data.T_state transforms from the condensed coordinates y_cond = [ X, th2, th3]
% to the expanded coordinates: y_exp = [ x2, y2, th2, x3, y3, th3]
data.T_state = zeros( 6, 3);
data.T_state( [1 3 4 6], :) = T_constrain;

% Mass Matrix

data.M = zeros( size(data.K));
data.M(1,1) = mass;

% Get the SDOF condensed m and k values for comparison
[data.m data.k] = condense( data.M, data.pre_yield_K);

% Now, add the "nodal mass" so that the mass matrix is nonsingular
data.M( 2:3,2:3) = diag (mass ./ mass_divisor);

% The name invMslab is legacy, should be changed to invM
data.invMslab = inv(data.M);

% Add the damping matrix -- 2 % on the first mode,
% 50% on the others since they are "not really there"
data.C = c_modal( data.M, data.pre_yield_K, [0.02, 0.5, 0.5]);
	
[T, xi, ev] = modaldata( data);
data.dyndata.T  =  T;
data.dyndata.xi = xi;
data.dyndata.ev = ev;
clear T xi ev

% Xss - ss refers to state space formulation
data.A = dif2ss( data.M, data.C, data.K);
data.Bss = [0 0 0 1/mass 0 0]';
data.Css = eye(6);
data.Dss = zeros(6,1);
data.pre_yield_A = dif2ss( data.M, data.C, data.pre_yield_K);

% Set up the nonlinear data to be used by nlodefile.
n = 1;  % element number of the current non-linear element
data.num_states = 0;
for x = 1 : data.num_elements
	etype = feval( data.elements(x).type, 'element-type');
	if isfield( etype, 'nonlinear')
		data.nl_el(n).num    = x;
		data.nl_el(n).data   = elget( x, data, 'nonlinear-data');
		% dofvec are the global dofs used by the element
		% -3 to account for the four fixed nodes at the base
		% TODO:  Figure out how to get the dofvec right without the kludge
		data.nl_el(n).dofvec = ...
			dofvec( data.elements(x), data.num_dofs, data.dofs_pos) - 3;
		data.state_beg(n)    = data.num_states + 1;
		data.state_end(n)    = data.num_states + ...
		feval( data.elements(x).type, 'num-states');
		data.num_states      = data.num_states + data.state_end(n) ...
			- data.state_beg(n) + 1;
		n = n + 1;
	end
end

% Set up the time and "acc_x" arrays, go past max_time for interp1
data.time = min_time : 0.4 * min(data.dyndata.T) : max_time+1;
data.acc_x = amplitude * sin( frequency * data.time);

% In case there are no nonlinear elements
if ~ isfield( data, 'nl_el')
	data.nl_el = [];
end

% Initial Conditions
y0 = zeros(size(data.M,1)*2 + data.num_states,1);

odeoptions = odeset( 'MaxStep', data.time(2) - data.time(1), ...
	'Vectorized', 'on')

% data.time goes beyond max_time so the interpolation in the
% ode file will work.  Cut it back for the linear problem.
idx_max = max( find( data.time <= max_time));
% compute the linear solution
initial_cputime = cputime;
data.Ylinear = lsim( data.pre_yield_A, data.Bss, data.Css, data.Dss, ...
	data.acc_x(1:idx_max), data.time(1:idx_max));
data.linear_total_run_time = cputime - initial_cputime;

initial_cputime = cputime;
[data.timer,data.response] = ode15s( 'nlodefile1d', [min_time max_time], y0, odeoptions, data);
data.ode_total_run_time = cputime - initial_cputime;
disp( ['Finished (uncontrolled): ' datestr(now)]);

linef('\n**  CPU time (seconds)  **\n\nNonlinear:  %f\nlinear:  %f\nRatio: %f\n', ...
	data.ode_total_run_time, data.linear_total_run_time, ...
	data.ode_total_run_time / data.linear_total_run_time);

data_uncontrolled = data;
% Add the damper
disp( 'Starting the passive case');

% The followin set the damping value so that the maximum force (from the linear
% case) in the x-direction is about 1/5 of the applied force.  I abandoned
% it because it only increased the damping coefficient to 3% (from 2%)
%Vmax_x = max( abs( data.Ylinear(:,4)));  % max velocity in x-direction
%divisor   = 5 * Vmax_x * cos( atan( height / width)) ^ 2;
% Because the damper is at an angle, the total capacity will actually
% be higher

% I set divisor so that the effective damping would be 5%
% Turns out this requires a damper capacity of about 40% of the applied force
% in the X-direction
divisor = 104.4;
data.materials(2).c       = amplitude / divisor;

data.elements(4).material = 2;
data.elements(4).section  = 1;  % Ignored
data.elements(4).nodes    = [ 1 3];
data.elements(4).type     = 'vdamper2d';
data.num_elements = length(data.elements);

data.C_elements = apply_bc( assemble( data, 'damping-matrix'), data, 'remove');
data.C_modal = data.C;
data.C = data.C_modal + T_constrain' * data.C_elements * T_constrain;
% Need to recompute A
data.A_uncontrolled = data.A;
data.A = dif2ss( data.M, data.C, data.K);

% compute the linear solution
temp = dif2ss( data.M, data.C, data.pre_yield_K);
initial_cputime = cputime;
data.YlinearC = lsim( temp, data.Bss, data.Css, data.Dss, ...
	data.acc_x(1:idx_max), data.time(1:idx_max));
data.linear_total_run_timeC = cputime - initial_cputime;

initial_cputime = cputime;
[data.timeC,data.responseC] = ode15s( 'nlodefile1d', [min_time max_time], y0, odeoptions, data);
data.ode_total_run_timeC = cputime - initial_cputime;
disp( ['Finished (controlled): ' datestr(now)]);

linef('\n**  CPU time (seconds)  **\n\nNonlinear:  %f\nlinear:  %f\nRatio: %f\n', ...
	data.ode_total_run_timeC, data.linear_total_run_timeC, ...
	data.ode_total_run_timeC / data.linear_total_run_timeC);

data.idx_max = idx_max;
clear idx_max

% The following allows use of other ODE solvers
% for x = 1 : 10
% initial_cputime = cputime;
% [r,c] = ode23t( 'nlodefile1d', [min_time max_time], y0, odeoptions, data);
% linef( 'Total run time(23t): %f', cputime - initial_cputime);
% end
% for x = 1 : 10
% initial_cputime = cputime;
% [r,c] = ode23tb( 'nlodefile1d', [min_time max_time], y0, odeoptions, data);
% linef( 'Total run time(23tb): %f', cputime - initial_cputime);
% end
% for x = 1 : 10
% initial_cputime = cputime;
% [r,c] = ode45( 'nlodefile1d', [min_time max_time], y0, odeoptions, data);
% linef( 'Total run time(45): %f', cputime - initial_cputime);
% end


% Do the plots
%figures( data);
